@@ -24,6 +24,7 @@ from bisect import bisect_left
 from threading import Thread, Lock
 import gobject
 import logging
+import time
 logging.basicConfig(level=logging.DEBUG)
 
 # AsyncItem object was borrowed from Joaquim Rocha's
@@ -112,6 +113,11 @@ class AsyncWorker(object):
             # tell queue about task done
             self._queue.task_done()
 
+            # Allow other threads to enter CPU. This does not really
+            # work really good due to the Globar Interpreter Lock, but
+            # what else can we do?
+            time.sleep(0)
+
     def add_task(self, task):
         self._queue.put(task)
 
