@@ -26,14 +26,16 @@ import gtk
 
 RSS_NAME = 'ReSiStance'
 RSS_COMPACT_NAME = 'resistance'
-RSS_VERSION = '0.8.7'
+RSS_VERSION = '0.9'
 RSS_DESCRIPTION = 'ReSiStance is a RSS feeds reader'
 RSS_URL = 'http://www.igalia.com'
 
 HOME_PATH = os.path.expanduser('~')
 RSS_CONF_FOLDER = os.path.join(HOME_PATH, '.osso/%s' % RSS_COMPACT_NAME)
 RSS_CONF_FILE = os.path.join(RSS_CONF_FOLDER, '%s.conf' % RSS_COMPACT_NAME)
-RSS_DB_FILE = os.path.join(RSS_CONF_FOLDER, '%s' % 'feeds.db')
+RSS_DB_DIR = os.path.join(RSS_CONF_FOLDER, 'feeds')
+OLD_RSS_DB_FILE = os.path.join(RSS_CONF_FOLDER, '%s' % 'feeds.db')
+RSS_DB_SUMMARY_FILE = os.path.join(RSS_CONF_FOLDER, '%s' % 'feed-summaries.db')
 
 DEFAULT_SYSTEM_APP_DIR = os.path.join(sys.prefix,
                                       'share',
