@@ -21,8 +21,9 @@
 #########################################################################
 
 from asyncworker import AsyncWorker, AsyncItem, PriorityQueue
+from time import time
+from utils import get_feed_id, get_feed_icon_uri, get_feed_subtitle
 from Queue import Queue
-import base64
 import constants
 import cPickle
 import feedparser
@@ -81,21 +82,42 @@ class LinkParser(SGMLParser):
 
 class ReSiStanceFeedDict(feedparser.FeedParserDict):
 
-    def __init__(self, feed_data, sync=None):
+    def __init__(self, feed_data):
         super(ReSiStanceFeedDict, self).__init__()
 
         self.update(feed_data)
 
-        if not 'visits' in self:
-            self['visits'] = 0
-
         # Initialize data
         for entry in self.entries:
             if not 'read' in entry:
                 entry['read'] = False
 
-        if not 'sync' in self:
-            self['sync'] = sync if sync != None else False
+        # Remove data from old versions
+        if 'visits' in self:
+            del self['visits']
+
+        if 'id' in self:
+            del self['id']
+
+        if 'sync' in self:
+            del self['sync']
+
+class ReSiStanceFeedSummary(dict):
+
+    def __init__(self, feed_id, href, title, subtitle, favicon, visits, unread, sync):
+        super(dict, self).__init__()
+
+        self.feed_id = feed_id
+        self.href = href
+        self.title = title
+        self.subtitle = subtitle
+        self.favicon = favicon
+        self.visits = visits
+        self.unread = unread
+        self.sync = sync
+
+    def update_from_feed_data(self, feed_data):
+        self.unread = len([entry for entry in feed_data.entries if entry.read == False])
 
 class FeedManager(gobject.GObject):
 
@@ -106,12 +128,14 @@ class FeedManager(gobject.GObject):
     def __init__(self, settings, conn_manager):
         super(FeedManager, self).__init__()
 
-        self.feed_data_list = []
-        self.settings = settings
+        self._settings = settings
+        self._feed_data_dict = {}
+        self._feeds_summary = {}
         self._conn_manager = conn_manager
         self._conn_manager.connect('connection-changed', self._on_connection_changed)
         self._google_reader_auth = None
         self._google_reader_auth_token = None
+        self._google_reader_expires = None
         self._async_worker = None
 
     def stop(self):
@@ -139,17 +163,14 @@ class FeedManager(gobject.GObject):
     def _on_connection_changed(self, conn_manager):
         pass
 
-    def get_feed_list(self):
-        return self.feed_data_list
-
     @requireWorker
-    def add_feed(self, url, sync, save, callback, data=None):
+    def add_feed(self, url, sync, callback, data=None):
         # Create a new worker to update from network
-        add_item = AsyncItem(self._add_feed_in_thread, (url,sync,save,),
+        add_item = AsyncItem(self._add_feed_in_thread, (url,sync),
                              callback, (data,))
         self._async_worker.add_task((self.DEFAULT_PRIORITY, add_item))
 
-    def _add_feed_in_thread(self, url, sync, save):
+    def _add_feed_in_thread(self, url, sync):
 
         parsed_url = urlparse.urlsplit(url)
         if parsed_url.scheme == '':
@@ -162,7 +183,7 @@ class FeedManager(gobject.GObject):
         if url == '':
             raise IOError('Invalid URL')
 
-        new_feed_data = ReSiStanceFeedDict(feedparser.parse(url), sync)
+        new_feed_data = ReSiStanceFeedDict(feedparser.parse(url))
 
         # 200 == OK, and 3xx are redirections.
         # On the other hand bozo==1 if there was some problem parsing the feed
@@ -172,73 +193,98 @@ class FeedManager(gobject.GObject):
             del new_feed_data
             raise IOError('Resource is not a feed')
 
-        self.feed_data_list.append(new_feed_data)
+        # Add data to hashtable
+        new_feed_id = get_feed_id(new_feed_data)
+        self._feed_data_dict[new_feed_id] = new_feed_data
+        self._feeds_summary[new_feed_id] = self._generate_summary(new_feed_data, sync)
 
         if 'link' in new_feed_data.feed:
             pixbuf = self._get_favicon_sync(new_feed_data.feed.link)
         else:
             pixbuf = self._get_favicon_sync(new_feed_data.href)
 
-        # Save to disk
-        if save:
-            try:
-                self.save(None)
-            except IOError:
-                pass
-
         return pixbuf, new_feed_data
 
     @requireWorker
-    def update_feed(self, feed, callback, data=None):
+    def update_feed(self, feed_id, callback, data=None):
         # Create a new worker to update from network
-        update_item = AsyncItem(self._update_feeds_in_thread, ([feed],),
+        update_item = AsyncItem(self._update_feed_in_thread, (feed_id,),
                                 callback, (data,))
         self._async_worker.add_task((self.DEFAULT_PRIORITY, update_item))
 
+    def _update_feed_in_thread(self, feed_id):
+
+        feed_data = self.get_feed_data(feed_id)
+        if not feed_data:
+            feed_data = self._load_in_thread(feed_id)
+            if not feed_data:
+                print 'ERROR: cannot load feed with ID: ' + string(feed_id)
+                return
+
+        try:
+            updated_feed_data = ReSiStanceFeedDict(feedparser.parse(feed_data.href))
+        except e:
+            return
+
+        # In case of network failure
+        if updated_feed_data == None or updated_feed_data.entries == None or \
+                len(updated_feed_data.entries) == 0:
+            return
+
+        updated_feed_date = updated_feed_data.feed.get('updated_parsed') or \
+            updated_feed_data.entries[0].get('updated_parsed')
+        feed_date = feed_data.feed.get('updated_parsed') or \
+            feed_data.entries[0].get('updated_parsed')
+
+        # Early return, nothing to do
+        if updated_feed_date <= feed_date:
+            return
+
+        # Use the entry link for those entries with no id attribute
+        old_entry_ids = [entry.id if 'id' in entry else entry.link \
+                             for entry in feed_data.entries]
+        updated_entry_ids = [entry.id if 'id' in entry else entry.link \
+                                 for entry in updated_feed_data.entries]
+        new_entries = [entry for entry in updated_feed_data.entries \
+                           if (entry.id if 'id' in entry else entry.link) not in old_entry_ids]
+        del old_entry_ids, updated_entry_ids
+
+        # Autodownload enclosures
+        if self._settings.auto_download:
+            to_download_urls = [entry.enclosures[0].href for entry in new_entries if 'enclosures' in entry]
+            to_download_paths = [self._settings.auto_download_folder+os.path.basename(urllib.url2pathname(url)) for url in to_download_urls]
+            self._download_items_in_thread(to_download_urls, to_download_paths)
+
+        if self._settings.delete_old:
+            feed_data.update(updated_feed_data)
+            feed_data.entries = new_entries
+        else:
+            # In order to keep read/unread status, just add
+            # new_entries to the old ones while updating the rest
+            # of the information
+            old_entries = feed_data.entries
+            feed_data.update(updated_feed_data)
+            feed_data.entries = old_entries + new_entries
+
     @requireWorker
-    def update_all(self, callback, data):
-        # Create a new worker to update from network
-        update_item = AsyncItem(self._update_feeds_in_thread, (self.feed_data_list,),
+    def remove_feed(self, feed_id, callback=None, data=None):
+        # Create a new worker to save the  file
+        remove_item = AsyncItem(self._remove_feed_in_thread, (feed_id,),
                                 callback, (data,))
-        self._async_worker.add_task((self.DEFAULT_PRIORITY, update_item))
+        self._async_worker.add_task((self.LOW_PRIORITY, remove_item))
 
-    def _update_feeds_in_thread(self, feed_data_list):
+    def _remove_feed_in_thread(self, feed_id):
 
-        for feed_data in feed_data_list:
-            updated_feed_data = ReSiStanceFeedDict(feedparser.parse(feed_data.href))
+        try:
+            os.remove(os.path.join(constants.RSS_DB_DIR, str(feed_id)))
+        except:
+            pass
 
-            # In case of network failure
-            if updated_feed_data == None or updated_feed_data.entries == None or \
-                    len(updated_feed_data.entries) == 0:
-                continue
+        del self._feeds_summary[feed_id]
+        if feed_id in self._feed_data_dict:
+            del self._feed_data_dict[feed_id]
 
-            updated_feed_date = updated_feed_data.feed.get('updated_parsed') or \
-                updated_feed_data.entries[0].get('updated_parsed')
-            feed_date = feed_data.feed.get('updated_parsed') or \
-                feed_data.entries[0].get('updated_parsed')
-
-            if updated_feed_date > feed_date:
-                old_entry_ids = [entry.id for entry in feed_data.entries]
-                updated_entry_ids = [entry.id for entry in updated_feed_data.entries]
-                new_entries = [entry for entry in updated_feed_data.entries if entry.id not in old_entry_ids]
-                del old_entry_ids, updated_entry_ids
-
-                # Autodownload enclosures
-                if self.settings.auto_download:
-                    to_download_urls = [entry.enclosures[0].href for entry in new_entries if 'enclosures' in entry]
-                    to_download_paths = [self.settings.auto_download_folder+os.path.basename(urllib.url2pathname(url)) for url in to_download_urls]
-                    self._download_items_in_thread(to_download_urls, to_download_paths)
-
-                # In order to keep read/unread status, just add
-                # new_entries to the old ones while updating the rest
-                # of the information
-                old_entries = feed_data.entries
-                # Do not overwrite visits, and sync values
-                feed_data.update(updated_feed_data, visits=feed_data['visits'], sync=feed_data['sync'])
-                feed_data.entries = old_entries + new_entries
-
-    def remove_feed(self, feed):
-        self.feed_data_list.remove(feed)
+        return True
 
     @requireWorker
     def export_opml(self, file_path, callback, data=None):
@@ -256,12 +302,12 @@ class FeedManager(gobject.GObject):
         root.setAttribute('version','1.0')
         root.appendChild(opml_doc.createElement('head'))
         body = opml_doc.createElement('body')
-        for feed_data in self.feed_data_list:
+
+        for summary in self._feeds_summary.values():
             outline = opml_doc.createElement('outline')
-            outline.setAttribute('title',feed_data.feed.title)
-            if 'subtitle' in feed_data.feed:
-                 outline.setAttribute('text',feed_data.feed.subtitle)
-            outline.setAttribute('xmlUrl',feed_data['href'])
+            outline.setAttribute('title',summary.title)
+            outline.setAttribute('text',summary.subtitle)
+            outline.setAttribute('xmlUrl',summary.href)
             body.appendChild(outline)
 
         root.appendChild(body)
@@ -320,8 +366,8 @@ class FeedManager(gobject.GObject):
         if self._google_reader_auth == None:
             try:
                 params = urllib.urlencode({"service": "reader",
-                                           "Email": self.settings.user,
-                                           "Passwd": self.settings.password})
+                                           "Email": self._settings.user,
+                                           "Passwd": self._settings.password})
                 url = constants.URL_LOGIN
                 content = urllib2.urlopen(url,params).read()
             except:
@@ -477,49 +523,52 @@ class FeedManager(gobject.GObject):
 
     @requireWorker
     def save(self, callback=None, data=None):
-        # TODO: migrate to "with" statement when available
-        try:
-            db_file = open(constants.RSS_DB_FILE, 'w')
-        except IOError:
-            print 'Cannot write to', constants.RSS_DB_FILE
-            raise
-        else:
-            # Create a new worker to store in disk
-            save_item = AsyncItem(self._save_in_thread, (db_file,),
-                                  callback, (data,))
-            self._async_worker.add_task((self.LOW_PRIORITY, save_item))
+        # Create a new worker to store in disk
+        save_item = AsyncItem(self.save_sync, (),
+                              callback, (data,))
+        self._async_worker.add_task((self.LOW_PRIORITY, save_item))
 
     def save_sync(self):
-        # TODO: migrate to "with" statement when available
-        try:
-            db_file = open(constants.RSS_DB_FILE, 'w')
-            cPickle.dump(self.feed_data_list, db_file)
-        except:
-            return
 
-    def _save_in_thread(self, db_file):
+        # First save summary, then feed data
         try:
-            cPickle.dump(self.feed_data_list, db_file)
-        except cPickle.UnpickleableError:
-            print 'Cannot serialize to', constants.RSS_DB_FILE
-            raise
+            self._save_feeds_summary()
+        except:
+            print 'Unable to save feeds summary'
+
+        for feed in self._feed_data_dict.values():
+            file_name = os.path.join(constants.RSS_DB_DIR, str(get_feed_id(feed)))
+
+            # TODO: migrate to "with" statement when available
+            try:
+                db_file = open(file_name, 'w')
+                cPickle.dump(feed, db_file)
+                db_file.close()
+            except:
+                print 'Could not save feed ' + feed.href
+                pass
 
     @requireWorker
-    def load(self, callback, data=None):
-        # TODO: migrate to "with" statement when available
-        try:
-            db_file = open(constants.RSS_DB_FILE, 'r')
-        except IOError:
-            print 'Cannot open', constants.RSS_DB_FILE
-            raise
-        else:
-            load_item = AsyncItem(self._load_in_thread, (db_file,),
-                                  callback, (data,))
-            self._async_worker.add_task((self.DEFAULT_PRIORITY, load_item))
+    def load(self, feed_id, callback, data=None):
+        load_item = AsyncItem(self._load_in_thread, (feed_id,),
+                              callback, (data,))
+        self._async_worker.add_task((self.DEFAULT_PRIORITY, load_item))
 
-    def _load_in_thread(self, db_file):
-        feed_data_list = cPickle.load(db_file)
-        self.feed_data_list = [ReSiStanceFeedDict(feed_data) for feed_data in feed_data_list]
+    def _load_in_thread(self, feed_id):
+        try:
+            db_file = open(os.path.join(constants.RSS_DB_DIR, str(feed_id)), 'r')
+            feed_data = cPickle.load(db_file)
+            db_file.close()
+        except IOError, e:
+            print e
+            # If the data does not exist reload it
+            summary = self._feeds_summary[feed_id]
+            feed_data = ReSiStanceFeedDict(feedparser.parse(summary.href))
+        except Exception:
+            return None
+
+        self._feed_data_dict[feed_id] = feed_data
+        return feed_data
 
     @requireWorker
     def get_favicon(self, url, callback, data=None):
@@ -534,7 +583,7 @@ class FeedManager(gobject.GObject):
         if os.path.exists(user_path) == False:
             os.makedirs(user_path, 0700)
 
-        file_name = os.path.join (user_path, base64.b64encode(url) + '.favicon.ico')
+        file_name = os.path.join (user_path, str(hash(url)) + '.ico')
         if os.path.exists(file_name) == False:
             parsed_url = urlparse.urlsplit(url)
             try:
@@ -575,15 +624,14 @@ class FeedManager(gobject.GObject):
         return pixbuf
 
     @requireWorker
-    def subscribe_feed_google(self, feed_data, feed_url, is_add_subcription, callback=None, user_data=None):
+    def subscribe_feed_google(self, feed_url, is_add_subcription, callback=None, user_data=None):
         # Create a new thread to synchronize(subscribe and unsubscribe)
-        subscribe_item = AsyncItem(self._subscribe_feed_google_in_thread, (feed_data,feed_url,is_add_subcription),
+        subscribe_item = AsyncItem(self._subscribe_feed_google_in_thread, (feed_url,is_add_subcription),
                                    callback, (user_data,))
         self._async_worker.add_task((self.LOW_PRIORITY, subscribe_item))
 
-    def _subscribe_feed_google_in_thread(self, feed_data, feed_url, is_add_subcription):
+    def _subscribe_feed_google_in_thread(self, feed_url, is_add_subcription):
         synced = False
-        feed_data.sync = is_add_subcription
 
         auth_headers, auth_token = self._authenticate_google_reader()
         if not auth_token:
@@ -615,6 +663,10 @@ class FeedManager(gobject.GObject):
         if not auth_token:
             return False
 
+        # Do not update if we have updated just a while ago
+        if time() < self._google_reader_expires:
+            return True
+
         try:
             # Get items from feed excluding those with read status. It
             # indeed really sucks that we can only ask for the unread
@@ -622,6 +674,10 @@ class FeedManager(gobject.GObject):
             unread_url = constants.URL_USER + constants.STATE_SUFFIX + 'reading-list' + '?' + 'xt=user/-/' + constants.STATE_SUFFIX + 'read' + '&n=1000'
             unread_request = urllib2.Request(unread_url, None, auth_headers)
             unread_response = urllib2.urlopen(unread_request)
+            # Set 10 minutes expiration time. Google returns some
+            # expiration time in unread_response.info()['expires'] but
+            # it's just some seconds.
+            self._google_reader_expires = time() + 10 * 60
             unread_data = unread_response.read()
         except:
             return False
@@ -652,11 +708,17 @@ class FeedManager(gobject.GObject):
             except:
                 return False
 
+        # Load all data. FIXME: there should be a better way
+        for summary in self._feeds_summary.values():
+            if summary.feed_id not in self._feed_data_dict:
+                feed_data = self._load_in_thread(summary.feed_id)
+
         for key in unread_dict.keys():
             # This could happen if there are unread items in feeds
             # that are not synchronized to ReSiStance. If that's the
             # case just ignore them
-            feed_data_list = [feed_data for feed_data in self.feed_data_list if feed_data.feed.link == key]
+            feed_data_list = [feed_data for feed_data in self._feed_data_dict.values() \
+                                  if feed_data.feed.link == key]
             if not feed_data_list:
                 continue
 
@@ -677,10 +739,84 @@ class FeedManager(gobject.GObject):
 
         # Mark all entries as read for feeds synced with Google Reader
         # with no unread items
-        sync_feeds = [feed for feed in self.feed_data_list if feed.sync and (feed.feed.link not in unread_dict)]
+        sync_feeds = []
+        for summary in self._feeds_summary.values():
+            if summary.sync:
+                feed_data = self._feed_data_dict[summary.feed_id]
+                if feed_data.feed.link not in unread_dict:
+                    sync_feeds.append(feed_data)
+
         for feed_data in sync_feeds:
             for entry in feed_data.entries:
                 if not entry.read:
                     entry.read = True
 
         return True
+
+    def get_feed_summary(self, feed_id):
+        return self._feeds_summary[feed_id]
+
+    def get_feed_data(self, feed_id):
+        try:
+            return self._feed_data_dict[feed_id]
+        except KeyError:
+            return None
+
+    @requireWorker
+    def load_feeds_summary(self, callback, data=None):
+        load_item = AsyncItem(self._load_feeds_summary_in_thread, (),
+                              callback, (data,))
+        self._async_worker.add_task((self.HIGH_PRIORITY, load_item))
+
+    def _generate_summary(self, feed_data, sync, visits=0):
+        feed_id = get_feed_id(feed_data)
+        unread_count = len([entry for entry in feed_data.entries if entry.read == False])
+        return ReSiStanceFeedSummary(feed_id, feed_data.href, feed_data.feed.title,
+                                     get_feed_subtitle(feed_data), get_feed_icon_uri(feed_data),
+                                     visits, unread_count, sync)
+
+    def _create_summary_from_old_storage(self):
+        db_file = open(constants.OLD_RSS_DB_FILE, 'r')
+        feed_data_list = cPickle.load(db_file)
+        db_file.close()
+
+        for feed_data in feed_data_list:
+            feed_id = get_feed_id(feed_data)
+            self._feeds_summary[feed_id] = self._generate_summary(feed_data, feed_data.sync, feed_data.visits)
+            self._feed_data_dict[feed_id] = ReSiStanceFeedDict(feed_data)
+
+    def _load_feeds_summary_in_thread(self):
+        try:
+            db_file = open(constants.RSS_DB_SUMMARY_FILE, 'r')
+            self._feeds_summary = cPickle.load(db_file)
+            db_file.close()
+        except IOError, e:
+            if e.errno != 2:
+                print e
+                return None
+            # IOError 2 means "file not found", so we're moving from
+            # an old version of ReSiStance that does not use summaries.
+            self._create_summary_from_old_storage()
+
+            # No need to keep the old storage. Try to save summaries
+            # and feed data in their new storages and if everything
+            # goes fine then remove the old db file
+            self.save_sync()
+            os.unlink(constants.OLD_RSS_DB_FILE)
+
+        except Exception, e:
+            print e
+            return None
+
+        return self._feeds_summary.values()
+
+    def _save_feeds_summary(self):
+        # Update summaries, some data might have changed
+        for feed_data in self._feed_data_dict.values():
+            summary_list = [summary for summary in self._feeds_summary.values() if summary.feed_id == get_feed_id(feed_data)]
+            if summary_list:
+                summary_list[0].update_from_feed_data(feed_data)
+
+        db_file = open(constants.RSS_DB_SUMMARY_FILE, 'w')
+        cPickle.dump(self._feeds_summary, db_file)
+        db_file.close()
