@@ -46,6 +46,8 @@ class Settings(object):
         self.password = ''
         self.sync_global = False
         self.entries_filter = constants.SHOW_ALL_FILTER
+        self.delete_old = False
+        self.auto_update_startup = False
 
     def load(self):
         if not (os.path.exists(constants.RSS_CONF_FILE) and
@@ -68,6 +70,8 @@ class Settings(object):
         self.password = self.config['password'] if 'password' in self.config else ''
         self.sync_global = self.config['sync_global'] if 'sync_global' in self.config else False
         self.entries_filter = self.config['entries_filter'] if 'entries_filter' in self.config else constants.SHOW_ALL_FILTER
+        self.delete_old = self.config['delete_old'] if 'delete_old' in self.config else False
+        self.auto_update_startup = self.config['auto_update_startup'] if 'auto_update_startup' in self.config else False
 
     def save(self):
         if self.config == None:
@@ -84,5 +88,7 @@ class Settings(object):
         self.config['password'] = self.password
         self.config['sync_global'] = self.sync_global
         self.config['entries_filter'] = self.entries_filter
+        self.config['delete_old'] = self.delete_old
+        self.config['auto_update_startup'] = self.auto_update_startup
 
         self.config.write()
